----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------

FS19PSUtil = {};
function FS19PSUtil.loadParticleSystemFromData(data, particleSystem, linkNodes, defaultEmittingState, defaultPsFile, baseDir, defaultLinkNode)
	if defaultLinkNode == nil then
		defaultLinkNode = linkNodes

		if type(linkNodes) == "table" then
			defaultLinkNode = linkNodes[1].node
		end
	end

	local linkNode = Utils.getNoNil(I3DUtil.indexToObject(linkNodes, data.nodeStr), defaultLinkNode)
	local psFile = data.psFile

	if psFile == nil then
		psFile = defaultPsFile
	end

	if psFile == nil then
		return
	end

	psFile = Utils.getFilename(psFile, baseDir)
	local rootNodeFile = loadI3DFile(psFile, true, true, false)
	local rootNode = rootNodeFile

	if rootNode == 0 then
		print("Error: failed to load particle system " .. psFile)

		return
	end

	if data.psRootNodeStr ~= nil then
		local newRootNode = I3DUtil.indexToObject(rootNode, data.psRootNodeStr)

		if newRootNode ~= nil then
			rootNode = newRootNode
		end
	else
		rootNode = getChildAt(rootNodeFile, 0)
	end

	if linkNode ~= nil then
		link(linkNode, rootNode)
	end

	local posX = data.posX
	local posY = data.posY
	local posZ = data.posZ

	if posX ~= nil and posY ~= nil and posZ ~= nil then
		setTranslation(rootNode, posX, posY, posZ)
	end

	local rotX = data.rotX
	local rotY = data.rotY
	local rotZ = data.rotZ

	if rotX ~= nil and rotY ~= nil and rotZ ~= nil then
		setRotation(rootNode, rotX, rotY, rotZ)
	end

	local returnValue = ParticleUtil.loadParticleSystemFromNode(rootNode, particleSystem, defaultEmittingState, data.worldSpace, data.forceFullLifespan, psFile)

	if rootNode ~= rootNodeFile then
		delete(rootNodeFile)
	end

	return returnValue
end;

function FS19PSUtil.loadParticleSystemFromNode(rootNode, particleSystem, defaultEmittingState, worldSpace, forceFullLifespan, file)
	if defaultEmittingState == nil then
		defaultEmittingState = true
	end

	if getHasClassId(rootNode, ClassIds.SHAPE) then
		local geometry = getGeometry(rootNode)

		if geometry ~= 0 and getHasClassId(geometry, ClassIds.PARTICLE_SYSTEM) then
			particleSystem.emitterShape = getEmitterShape(geometry)
			particleSystem.emitterShapeSize = getEmitterSurfaceSize(geometry)
			particleSystem.defaultEmitterShapeSize = getEmitterSurfaceSize(geometry)

			if worldSpace then
				local parent = getParent(rootNode)

				if particleSystem.emitterShape ~= 0 and getParent(particleSystem.emitterShape) == rootNode then
					local x, y, z = getScale(particleSystem.emitterShape)

					setTranslation(particleSystem.emitterShape, worldToLocal(parent, getWorldTranslation(particleSystem.emitterShape)))

					local dx, dy, dz = worldDirectionToLocal(rootNode, localDirectionToWorld(particleSystem.emitterShape, 0, 0, 1))
					local upx, upy, upz = worldDirectionToLocal(rootNode, localDirectionToWorld(particleSystem.emitterShape, 0, 1, 0))

					setDirection(particleSystem.emitterShape, dx, dy, dz, upx, upy, upz)
					link(parent, particleSystem.emitterShape)
					setScale(particleSystem.emitterShape, x, y, z)
				end

				link(getRootNode(), rootNode)
				setTranslation(rootNode, 0, 0, 0)
				setRotation(rootNode, 0, 0, 0)
			end

			setObjectMask(rootNode, 16711807)

			particleSystem.geometry = geometry
			particleSystem.shape = rootNode
			particleSystem.worldSpace = worldSpace
			particleSystem.forceFullLifespan = forceFullLifespan
			particleSystem.originalLifespan = getParticleSystemLifespan(geometry)

			setEmittingState(geometry, defaultEmittingState)
		end
	end

	particleSystem.isEmitting = defaultEmittingState

	return rootNode
end;