----------------------------------------------------------------------------
-- @Author: ViperGTS96------------------------------------------------------
----------------------------------------------------------------------------
--------------------"The simplest design is the best design." --------------
----------------------------------------------------------------------------
----------------------------------------------------------------------------
instantWash = {};
instantWash.modDirectory  = g_currentModDirectory;
source(Utils.getFilename("fs19PSUtil.lua", instantWash.modDirectory));
local modDescFile = loadXMLFile("modDesc", instantWash.modDirectory .. "modDesc.xml");
instantWash.title = getXMLString(modDescFile, "modDesc.title.en");
instantWash.author = getXMLString(modDescFile, "modDesc.author");
instantWash.version = getXMLString(modDescFile, "modDesc.version");
delete(modDescFile);
instantWash.inputKey = Input.KEY_insert;
instantWash.washSound = createSample("washSound");
loadSample(instantWash.washSound, instantWash.modDirectory.."washSound.ogg", false);

function instantWash.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Washable, specializations);
end;

function instantWash.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", instantWash);
	SpecializationUtil.registerEventListener(vehicleType, "onLeaveVehicle", instantWash);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", instantWash);
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", instantWash);
end;

function instantWash.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "sendInstantWashEvent", instantWash.sendInstantWashEvent);
end;

function instantWash:onLoad(savegame)
	self.spec_instantWash = {};
	local spec = self.spec_instantWash;
	spec.instantWashEvents = {};
	spec.instantWashEventIds = {};	
	spec.washTime = 0;
	spec.washEffect = {};
	spec.washEffect.isActive = false;
	local scaleFactor = 1.0;
	local y = 1.8;
	if self.spec_wheels ~= nil and #self.spec_wheels.wheels > 0 then
		y = y*2; 
		if self.size.height ~= nil then y = self.size.height; y = y-0.25; end;
	end;
	local psData = {};
	psData.psFile = "particleSystems/wfxSplashes.i3d";
	psData.posX = 0;
	psData.posY = y;
	psData.posZ = 1.0;
	psData.worldSpace = false;
	local loaded = FS19PSUtil.loadParticleSystemFromData(psData, spec.washEffect, nil, false, nil, instantWash.modDirectory, self.components[1].node);
	setScale(spec.washEffect.shape, scaleFactor, scaleFactor, scaleFactor);
end;

function instantWash:onUpdate(dt)
	if self.isServer then
		local vehicle = self:getRootVehicle();
		local spec = self.spec_instantWash;
		local rootSpec = vehicle.spec_instantWash;
		if rootSpec.washEffect ~= nil and rootSpec.washEffect.isActive then
			if rootSpec.washTime > 0 then
				for _, nodeData in ipairs(self.spec_washable.washableNodes) do
					self:setNodeDirtAmount(nodeData, 0, true);
					ParticleUtil.setEmittingState(spec.washEffect, true);
				end;
				rootSpec.washTime = rootSpec.washTime-1;
			else
				rootSpec.washEffect.isActive = false;
			end;
		else
			ParticleUtil.setEmittingState(spec.washEffect, false);
			rootSpec.washTime = 0;
		end;
	end;
end;


function instantWash:onLeaveVehicle()
	if self.isServer then
		local spec = self.spec_instantWash;
		if spec.washEffect ~= nil then
			if spec.washEffect.isActive or spec.washTime > 0 then
				ParticleUtil.setEmittingState(spec.washEffect, false);
				spec.washEffect.isActive = false;
				spec.washTime = 0;
			end;
		end;
	end;
end;

function instantWash:toggleActive(actionName, keyStatus, arg3, arg4, arg5)
	if actionName == "instantWash_washKey" then
		local spec = self:getRootVehicle().spec_instantWash;
		if not spec.washEffect.isActive then
			if not isSamplePlaying(instantWash.washSound) then playSample(instantWash.washSound,1,1,0,0,0); end;
		end;
		local washTime = math.min(spec.washTime+2, 10);
		self:sendInstantWashEvent(true, washTime);
	end;
end;

function instantWash:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.isClient then
		local spec = self.spec_instantWash;
		self:clearActionEventsTable(spec.instantWashEvents);
		spec.instantWashEventIds = {};
        if self:getIsActiveForInput(true, true) then
			local _, actionEventId = self:addActionEvent(spec.instantWashEvents, 'instantWash_washKey', self, instantWash.toggleActive, false, true, true, true, nil);
			g_inputBinding:setActionEventTextVisibility(actionEventId, false);
			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_LOW);
			table.insert(spec.instantWashEventIds, actionEventId);
		end;
	end;
end;

function instantWash:sendInstantWashEvent(active, washtime, noEventSend)
	instantWash_Event.sendEvent(self, active, washtime, noEventSend);
	local spec = self:getRootVehicle().spec_instantWash;
	spec.washEffect.isActive = active;
	spec.washTime = washtime;
end;

print(instantWash.title.." : v"..instantWash.version.." by "..instantWash.author.." activated");

instantWash_Event = {};
instantWashEvent_mt = Class(instantWash_Event, Event);
InitEventClass(instantWash_Event,"instantWashEvent");

function instantWash_Event:emptyNew()
    local self = Event.new(instantWashEvent_mt);
    self.className = "instantWashEvent";
    return self;
end;
function instantWash_Event:new(vehicle,state,washtime)
	local self = instantWash_Event.emptyNew()
    self.vehicle = vehicle;
    self.state = state;
    self.washtime = washtime;
    return self;
end;
function instantWash_Event:readStream(streamId,connection)
    self.vehicle = NetworkUtil.readNodeObject(streamId);
    self.state = streamReadBool(streamId);
    self.washtime = streamReadInt8(streamId);
    self:run(connection);
end;
function instantWash_Event:writeStream(streamId,connection)
    NetworkUtil.writeNodeObject(streamId,self.vehicle);
    streamWriteBool(streamId,self.state);
    streamWriteInt8(streamId,self.washtime);
end;
function instantWash_Event:run(connection)
    self.vehicle:sendInstantWashEvent(self.state, self.washtime, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(instantWash_Event:new(self.vehicle,self.state, self.washtime),nil,connection,self.vehicle);
    end;
end;
function instantWash_Event.sendEvent(vehicle,state,washtime, noEventSend)
    if noEventSend == nil or noEventSend == false then
        if g_server ~= nil then
            g_server:broadcastEvent(instantWash_Event:new(vehicle,state,washtime),nil,nil,vehicle);
        else
            g_client:getServerConnection():sendEvent(instantWash_Event:new(vehicle,state,washtime));
        end;
    end;
end;